/*
 * Copyright 2010 Tilburg University
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package nl.uvt.pdproxy;

import java.io.*;
import java.util.*;

import nu.xom.*;

public class FileSystemStore implements IStore {

  private File directory;
  static Builder builder = new Builder();
  
  public FileSystemStore(File directory) {
    this.directory = directory;  
  }
  
  public File getDirectory() {
    return this.directory;
  }
  
  public List<String> getPersonIds()
  throws StoreException {
    List<String> list = new ArrayList<String>();
    File[] files = this.directory.listFiles();
    for (int i=0; i<files.length; i++) {
      String personId = files[i].getName();
      list.add(personId);
    }
    return list;
  }

  public Person getPerson(String id)
  throws UnknownObjectException, StoreException {
    Person person = null;
    File file = this.getFile(id);
    if (! file.exists()) {
      throw new UnknownObjectException("Cannot find file "+file.getAbsolutePath());
    }
    try {
      InputStream is = new FileInputStream(file);
      Document document = builder.build(is);
      person = new Person(id, document);
    } catch (IOException e) {
      throw new StoreException("IOException encountered while reading file "+file.getAbsolutePath());
    } catch (ValidityException e) {
      throw new StoreException("ValidityException encountered while parsing file "+file.getAbsolutePath());
    } catch (ParsingException e) {
      throw new StoreException("ParsingException encountered while parsing file "+file.getAbsolutePath());
    }
    return person;
  }
  
  public long getPersonFileAge(String id)
  throws UnknownObjectException, StoreException {
    File file = this.getFile(id);
    if (! file.exists()) {
      throw new UnknownObjectException("Cannot find file "+file.getAbsolutePath());
    }
    Date now = new Date();
    return now.getTime() - file.lastModified();
  }

  public void storePerson(Person person, boolean allowOverwrite)
  throws ObjectAlreadyExistsException, StoreException {
    File file = this.getFile(person.getId());
    if (file.exists() && allowOverwrite == false) {
      throw new ObjectAlreadyExistsException("File "+file.getAbsolutePath()+" already exists and overwriting not allowed");
    }
    try {
      FileOutputStream os = new FileOutputStream(file);
      Serializer serializer = new Serializer(os);
      serializer.write(person.getFoaf());
      serializer.flush();
    } catch (IOException e) {
      throw new StoreException("IOException encountered while writing file "+file.getAbsolutePath());
    }
  }

  public void removePerson(String id)
  throws UnknownObjectException, StoreException {
    File file = this.getFile(id);
    if (! file.exists()) {
      throw new UnknownObjectException("Cannot find file "+file.getAbsolutePath());
    }
    if (file.delete() == false) {
      throw new StoreException("Cannot remove file "+file.getAbsolutePath());
    }
  }
  
  private File getFile(String id) {
    return new File(this.directory, id);
  }

}
