/*
 * Copyright 2010 Tilburg University
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package nl.uvt.pdproxy;

import java.util.*;
import nu.xom.*;
import org.apache.log4j.*;

public class Proxy {
  private ISource source; // Experts and Expertise
  private IStore cache; // Temporary storage
  private IStore store; // Persistent storage for persons that have left
  private long maxAgeInCache; // Max lifetime in cache in ms
  public static long DEFAULT_MAX_AGE_IN_CACHE =  1000 * 60 * 60 * 12;
  //                                                              ^^
  //                                                              hours
  private long minAgeToStore; // When do we move a person that cannot be generated to store in ms
  public static long DEFAULT_MIN_AGE_TO_STORE =  1000 * 60 * 60 * 24 * 14;
  //                                                                   ^^
  //                                                                   days
  
  private static Logger logger = Logger.getLogger(Proxy.class);

  public Proxy() {
    this.maxAgeInCache = DEFAULT_MAX_AGE_IN_CACHE;
    this.minAgeToStore = DEFAULT_MIN_AGE_TO_STORE;
  }
  
  public void setSource(ISource source) { this.source = source; }
  
  public IStore getCache() { return this.cache; }
  public void setCache(IStore cache) { this.cache = cache; }

  public IStore getStore() { return this.store; }
  public void setStore(IStore store) { this.store = store; }

  public long getMaxAgeInCache() { return this.maxAgeInCache; }
  public void setMaxAgeInCache(long maxAgeInCache) { this.maxAgeInCache = maxAgeInCache; }

  public long getMinAgeToStore() { return this.minAgeToStore; }
  public void setMinAgeToStore(long minAgeToStore) { this.minAgeToStore = minAgeToStore; }
  
  public Document listCache()
  throws UnknownObjectException, StoreException {
    return exportDocument(this.cache);
  }

  public Document listStore()
  throws UnknownObjectException, StoreException {
    return exportDocument(this.store);
  }
  
  public Document getFoaf(String idString, boolean withOrganizationInfo) {

    Document document = null;
    
    /*
     * Try to get the person from the store first:
     */
    try {
      Person person = this.store.getPerson(idString);
      document = person.getFoaf();
      logger.info("Returning data for person with id = '"+idString+"' from store");
    } catch (Exception e) {
      logger.debug("Cannot get person from store. Msg="+e.getMessage());
      // Get person from cache or source here?
    }

    if (document == null) {
      /*
       * Not in store. Check cache:
       */
      try {
        long age = this.cache.getPersonFileAge(idString);
        logger.debug("Age of '"+idString+"' in cache: "+age);
        if (age <= this.maxAgeInCache) {
          Person person = this.cache.getPerson(idString);
          document = person.getFoaf();
          logger.info("Returning data for person with id = '"+idString+"' from cache");
        }
        else {
          logger.debug("Data in cache for person with id = '"+idString+"' too old");
        }
      } catch (Exception e) {
        logger.debug("Cannot get person from cache. Msg="+e.getMessage());
      }
    }

    if (document == null) {
      /*
       * Not in cache or data in cache too old:
       */
      try {
        /*
         * Generate fresh document from source:
         */
        Person person = this.source.getPerson(idString);
        document = person.getFoaf();
        /*
         * ...and store in cache:
         */
        this.cache.storePerson(person, true);
        logger.info("Stored new data for id = '"+idString+"' in the cache");

      } catch (Exception e) {
        logger.error("Cannot generate person from source or store in cache. Msg="+e.getMessage());
      }
    }

    if (document == null) {
      /*
       * If document is null, but there is a person available in cache, the data
       * in the cache is too old. Return this person anyway.
       * If the data is really old, we assume the person is not anymore available
       * in the source and has left. Now move the data from cache to store for permanent
       * storage:
       */
      try {
        Person person = this.cache.getPerson(idString);
        document = person.getFoaf();
        long age = this.cache.getPersonFileAge(idString);
        if (age > this.minAgeToStore) {        
          this.store.storePerson(person, true);
          this.cache.removePerson(idString);
          logger.info("Moved data for id = '"+idString+"' from cache to store");

          // TODO: Mail people that person x has apparently left and that the data is now in the store 

        }
      } catch (Exception e) {
        String msg = "Cannot get or transform document for person with id='"+idString+"'. Msg="+e.getMessage();
        logger.error(msg);
      }
      
      if (document == null) {
        String msg = "Cannot get or transform document for person with id='"+idString+"'";
        Element rootElement = new Element("diagnostic");
        rootElement.appendChild(msg);
        document = new Document(rootElement);
        
        return document; // Make sure that below we only have foaf documents
      }
    }
    
    if (document != null && withOrganizationInfo == false) {
      Element rootElement = document.getRootElement();
      Nodes nodes = rootElement.query("foaf:Organization", nl.uvt.commons.xml.Namespaces.getContext());
      for (int n=nodes.size()-1; n>=0; n--) {
        // logger.debug("Removing element "+((Element)nodes.get(n)).getLocalName()+" from response");
        nodes.get(n).detach();
      }
    }

    return document;
  }

  private static Document exportDocument(IStore store)
  throws UnknownObjectException, StoreException {
    Element rootElement = new Element("store");
    List<String> list = store.getPersonIds();
    for (String personId : list) {
      Element personElement = new Element("person");
      Attribute idAttribute = new Attribute("id", personId);
      personElement.addAttribute(idAttribute);
      Attribute ageAttribute = new Attribute("age", String.format("%d", store.getPersonFileAge(personId)));
      personElement.addAttribute(ageAttribute);
      rootElement.appendChild(personElement);
    }
    return new Document(rootElement);
  }
  
}
