/*
 * Copyright 2010 Tilburg University
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package nl.uvt.pdproxy;

import java.io.*;
import javax.servlet.ServletException;
import javax.servlet.http.*;
import java.util.*;
import nu.xom.*;
import org.apache.log4j.*;

public class Servlet extends HttpServlet {
  private static final long serialVersionUID = 7127313633745553489L;

  private Proxy proxy;
  private Logger logger;
  
  private static Builder builder = new Builder();

  @Override
  public void init() {

    /*
     * Setup directories list where we try to find configuration files:
     */
    List<File> configDirectories = new ArrayList<File>();
    String configPath = getServletContext().getInitParameter("configPath");
    System.out.println("configPath = '"+configPath+"'");
    if (configPath != null && configPath.length() > 0) {
      String[] directoryNames = configPath.split(File.pathSeparator);
      for (int i=0; i<directoryNames.length; i++) {
        StringBuffer directoryName = new StringBuffer(directoryNames[i]);
        if (directoryName.substring(0, 2).equals("~"+File.separator)) {
          // Replace '~' with home directory
          directoryName.replace(0, 1, System.getProperty("user.home"));
        }
        File directory = new File(directoryName.toString());
        if (directory.exists()) {
          configDirectories.add(directory);
        }
      }
    }
    
    Configuration configuration = new Configuration(configDirectories);
    configuration.loadProperties();
    configuration.setupLog4JConfiguration();
    
    this.logger = Logger.getLogger(Servlet.class);

    /*
     * Create and setup the proxy:
     */
    this.proxy = new Proxy();

    try {
      String s;
      
      this.logger.info("Setting up PDProxy");
      
      /*
       * Setup source:
       */
      s = configuration.getProperty(Configuration.EXPERTS_AND_EXPERTISE_URL_TEMPLATE);
      if (s == null) {
        throw new ConfigurationException(
            "Missing configuration setting '"+
            Configuration.EXPERTS_AND_EXPERTISE_URL_TEMPLATE+
            "'");
      }
      logger.info("Experts and Expertise URL template = '"+s+"'");

      InputStream is = null;
      File xslFile = configuration.getFile("experts-and-expertise2foaf.xsl");
      if (xslFile != null) {
        is = new FileInputStream(xslFile);
      }
      else {
        is = this.getClass().getClassLoader().getResourceAsStream("experts-and-expertise2foaf.xsl");
      }
      Document xslDocument = builder.build(is);
      ISource source = new ExpertsAndExpertiseSource(s, xslDocument);
      this.proxy.setSource(source);
      
      /*
       * Setup cache and store:
       */
      IStore cache = getFileSystemStore(configuration, Configuration.CACHE_DIRECTORY); 
      this.proxy.setCache(cache);
      this.logger.info("Cache directory = "+((FileSystemStore)cache).getDirectory().getAbsolutePath());
      
      IStore store = getFileSystemStore(configuration, Configuration.STORE_DIRECTORY);
      this.proxy.setStore(store);
      this.logger.info("Store directory = "+((FileSystemStore)store).getDirectory().getAbsolutePath());
      
      s = configuration.getProperty(Configuration.CACHE_MAX_AGE);
      if (s != null) {
        this.proxy.setMaxAgeInCache(Integer.parseInt(s));
      }
      this.logger.info("Max age in cache = "+this.proxy.getMaxAgeInCache()+" ms");
      s = configuration.getProperty(Configuration.STORE_MIN_AGE);
      if (s != null) {
        this.proxy.setMinAgeToStore(Integer.parseInt(s));
      }
      this.logger.info("Min age for store = "+this.proxy.getMinAgeToStore()+" ms");
      
      this.logger.info("Item count in cache = "+this.proxy.getCache().getPersonIds().size());
      this.logger.info("Item count in store = "+this.proxy.getStore().getPersonIds().size());

    } catch (Exception e) {
      this.logger.fatal("Cannot initialize servlet. Msg="+e.getMessage());
      e.printStackTrace();
    }
  }
  
  @Override
  public void doGet(HttpServletRequest request, HttpServletResponse response)
  throws ServletException, IOException {
    
    OutputStream outputStream = response.getOutputStream();
    Serializer serializer = new Serializer(outputStream, "UTF-8");
    serializer.setIndent(2);
    response.setContentType("text/xml");
    Document returned = handleRequest(request);
    if (returned != null) {
      serializer.write(returned);
    }
  }
  
  private Document handleRequest(HttpServletRequest request) {
    Document document = null;

    String requestString = request.getParameter("request");
    if (requestString == null)
      requestString = "";

    if (requestString.equals("listcache")) {
      try {
        document = this.proxy.listCache();
      } catch (Exception e) {
        document = makeExceptionDocument(e);
      }
    }
    else if  (requestString.equals("liststore")) {
      try {
        document = this.proxy.listStore();
      } catch (Exception e) {
        document = makeExceptionDocument(e);
      }
    }
    else if (requestString.equals("getfoaf")) {
      String idString = request.getParameter("id");
      try {
        String withOrganizationInfoString = request.getParameter("withOrganizationInfo");
        boolean withOrganizationInfo = true;
        if (withOrganizationInfoString != null && withOrganizationInfoString.equals("false")) {
          withOrganizationInfo = false;
        }
        document = this.proxy.getFoaf(idString, withOrganizationInfo);
      } catch (Exception e) {
        document = makeExceptionDocument(e);
      }
    }
    else {
      document = makeExceptionDocument("Unknown request");
    }

    return document;
  }

  private IStore getFileSystemStore(Configuration configuration, String property)
    throws ConfigurationException {
    String s = configuration.getProperty(property);
    if (s == null) {
      throw new ConfigurationException(
          "Missing configuration setting '"+property+"'");
    }
    File f = new File(s);
    if (! f.exists()) {
      throw new ConfigurationException(
          "Configuration setting '"+property+"' points to directory that does not exist: '"+s+"'");
    }
    return new FileSystemStore(f);
  }

  // Ugly
  private static Document makeExceptionDocument(String message) {
    Element rootElement = new Element("exception");
    Attribute exceptionAttribute = new Attribute("class", "java.lang.Exception");
    rootElement.addAttribute(exceptionAttribute);
    rootElement.appendChild(message);
    return new Document(rootElement);
  }

  private static Document makeExceptionDocument(Exception e) {
    Element rootElement = new Element("exception");
    Attribute exceptionAttribute = new Attribute("class", e.getClass().getName());
    rootElement.addAttribute(exceptionAttribute);
    rootElement.appendChild(e.getMessage());
    return new Document(rootElement);
  }
}

